import { Observable, Subject } from "rxjs";
import { AdaptorTransmissionRole, TransportEvent } from "./connector.interface";
import { MessageTransmissionTransmitter } from "../transmission/msg.transmission.transmitter";
import { MessageTransmissionReceiver } from "../transmission/msg.transmission.receiver";
import { RetransmissionService } from "../utils/retransmission.service";
import { Adapter } from "../connector/adapter.base";
export interface MessageTransmissionManager {
    subscribe(): Observable<MessageTransmissionSet>
    getEvent(): Observable<TransportEvent>
}

export interface MessageTransmissionSet {
    id: string,
    transmitter: MessageTransmissionTransmitter,
    receiver: MessageTransmissionReceiver,
    requestResponse: MessageRequestResponse,
    event: Observable<TransportEvent>
}

export interface MessageTransmissionBase {
    transmissionRole: AdaptorTransmissionRole
    mainAdapter: Adapter

    getInfo(): TransmissionProfile
    setUpAdapter(adapter: Adapter): void
}

export interface MessageReceiver extends MessageTransmissionBase {
    receiverProfile: ReceiverProfile

    getMessageBus(bus: Bus): Observable<any>
    setReceiver(receiverProfile: ReceiverProfile, event: Observable<TransportEvent>): void
}

export interface MessageTransmitter extends MessageTransmissionBase {
    transmitterProfile: TransmitterProfile
    retransmission: RetransmissionService

    setTransmitter(transmitterProfile: TransmitterProfile, event: Observable<TransportEvent>): void
}

export interface MessageRequestResponse extends MessageTransmissionBase {
    transmitterInstance: MessageTransmissionTransmitter
    receiverInstance: MessageTransmissionReceiver

    setTransmissionProfile(transmissionInfo: MessageTransmissionTransmitter, receiverInfo: MessageTransmissionReceiver): void
}

export interface FisMessage {
    header: {
        messageID: string,
        messageName: `NotificationMessage` | `ResponseMessage` | `RequestMessage`
    },
    data: any
}

export interface TransmissionProfile {
    id: string,
    name: string,
    dateCreated: Date
}

export interface TransmitterProfile extends TransmissionProfile {
}

export interface ReceiverProfile extends TransmissionProfile {
}

export interface RequestResponseProfile extends TransmissionProfile {

}

export enum Bus {
    GeneralBus,
    ResponseMessageBus,
    ErrorMessageBus,
    NotificationMessageBus
}

export interface EventMessage {
    clientId: string,
    message: string,
    payload?: any
}